{
 ******************** DBUT DE L'OBJET CONJUGATEURS ********************

   Permet de charger en mmoire plusieurs suites de suffixes, distingues par
des Cls (chanes de caractres distinctes), puis de les invoquer au besoin.

PRREQUIS:

   -TYPES

   L'usager doit fournir un type TChaine du genre chane de caractres, assez
grand pour contenir les codes des conjugateurs et les suffixes.  Il doit
galement fournir un type TNomFichier, du mme genre, assez grand pour contenir
un nom de fichier.

USAGE:

   CRATION/DESTRUCTION

   P Creer_Cnjgtrs (VAR C : TOCnjgtrs,
		    Nom   : TNomFichier)

   Cre l'objet vide, puis le remplit.  Le fichier Nom contient des paires
Cl-Fichier, chaque Fichier donnant l'numeration des suffixes du conjugateur
Cl.

   P Detruire_Cnjgtrs (VAR C : TOCnjgtrs)	Dtruit l'objet.

   CONSULTATION

   P Trouver_Cnjgtrs (VAR C : TOCnjgtrs,
		      Cle   : TChaine)
						
   Trouve le conjugateur rpondant  la Cl; la Cl DOIT avoir t charge
auparavant par Lire_Cnjgtrs.

   F Suffixe_Cnjgtrs (VAR C       : TOCnjgtrs,
		      VAR Reponse : TChaine   ) : BOOLEAN

   S'il reste des suffixes dans le conjugateur, rpond VRAI et fournit le
suffixe suivant dans Reponse; sinon, rpond FAUX.

NOTES:

   L'objet est constitu d'une liste chane de conjugateurs.  Chaque
conjugateur est lui-mme une liste chaine de suffixes.

SIGN:

   Daniel U. Thibault     78 840 046
   13 Dcembre 1988
}

{ ++++++++++++++++++++ PROCDURES VISIBLES ++++++++++++++++++++ }

{ %%% CRATION/DESTRUCTION %%% }

PROCEDURE Creer_Cnjgtrs (VAR C      : TOCnjgtrs;
                         Nom_Source : TNomFichier);
{
   Cre l'objet vide, puis le remplit grce au fichier de nom Nom_Source.

Pr:  C n'existe pas encore (sinon l'usager perdra l'accs  une partie de la
      mmoire);
      Nom_Source dsigne un fichier-texte contenant un certain nombre de paires
      de lignes (la premire ligne de chaque paire donne la Cl du conjugateur,
      la seconde le nom du fichier-texte contenant les suffixes).
      Nom_Source et les fichiers qu'il dsigne sont tous non-vides.

Post: C contient les conjugateurs dcrits par Nom_Source.
}
VAR
   NomFichier : TNomFichier;   { Nom du fichier contenant les suffixes }
   Liste,                      { contient les paires Cle-NomFichier }
   Source     : TEXT;          { contient les suffixes }
   Cle,                        { Cle du conjugateur }
   Suffixe    : TChaine;       { Suffixe }

BEGIN { Crer_Cnjgtrs }
   C.Tete        := NIL;
   C.Conjugateur := NIL;
   C.Suffixe     := NIL;

   OPEN (Liste, Nom_Source, HISTORY := READONLY);
   RESET (Liste);
   REPEAT
{
 Q : Les paires Cle-Fichier prcdentes ont t traites, et
     C.Conjugateur pointe au dernier Conjugateur cr
}
      READLN (Liste, Cle);
      READLN (Liste, NomFichier);
      OPEN (Source, NomFichier, HISTORY := READONLY);
      RESET (Source);
      WRITELN ('Lecture du conjugateur "', Cle, '"');

      IF C.Tete = NIL
      THEN BEGIN
         NEW (C.Tete);
         C.Conjugateur := C.Tete;
      END { THEN }
      ELSE BEGIN
         NEW (C.Conjugateur^.Suivant);
         C.Conjugateur := C.Conjugateur^.Suivant;
      END; { IF }
{
   C.Conjugateur pointe au Conjugateur  crer
}
      C.Conjugateur^.Cle := Cle;
      C.Suffixe := NIL;
{
   Lecture des suffixes
}
      REPEAT
{
   C.Suffixe pointe au dernier suffixe cr
}
         IF C.Suffixe = NIL
         THEN BEGIN
{
   Premier suffixe
}
            NEW (C.Conjugateur^.Premier);
            C.Suffixe := C.Conjugateur^.Premier;
         END { THEN }
         ELSE BEGIN
{
   Suffixes suivants
}
            NEW (C.Suffixe^.Suivant);
            C.Suffixe := C.Suffixe^.Suivant;
         END; { IF }
{
   C.Suffixe pointe au suffixe  lire
}
         READLN (Source, C.Suffixe^.Suffixe);
      UNTIL EOF (Source);
      C.Suffixe^.Suivant := NIL;
      CLOSE (Source);
   UNTIL EOF (Liste);
   C.Conjugateur^.Suivant := NIL;
   CLOSE (Liste);
   WRITELN ('Lectures effectues');
END; { Crer_Cnjgtrs }

PROCEDURE Detruire_Cnjgtrs (VAR C : TOCnjgtrs);
{
   Dtruit l'objet C.

Pr:  C existe.

Post: C est vide.
}
VAR
   P : TPntrSuffixe;

BEGIN { Dtruire_Cnjgtrs }
   C.Conjugateur := C.Tete;
   WHILE C.Tete <> NIL DO BEGIN
{
   C est un objet Conjugateurs bien form, non-vide
   Tete = Conjugateur
   Autrement dit, les Conjugateurs prcdant la Tete courante ont t dtruits
}
      C.Tete := C.Tete^.Suivant;
{
   Dtruire les suffixes de Conjugateur
}
      C.Suffixe := C.Conjugateur^.Premier;
      WHILE C.Suffixe <> NIL DO BEGIN
{
   Les suffixes prcdant celui point ont t dtruits
}
         P := C.Suffixe;
         C.Suffixe := C.Suffixe^.Suivant;
         DISPOSE (P);
      END; { WHILE }
{
   Les suffixes du Conjugateur point ont t dtruits
}
      DISPOSE (C.Conjugateur);
      C.Conjugateur := C.Tete;
   END; { WHILE }
END; { Dtruire_Cnjgtrs }

{ %%% CONSULTATION %%% }

PROCEDURE Trouver_Cnjgtrs (VAR C : TOCnjgtrs;
                           Cle   : TChaine);
{
   Trouve le conjugateur rpondant  la Cle.

Pr:  C existe et a t lu (est non-vide).
      Cle est prsent dans C.

Post: Le curseur de l'objet (C) est prt  subir les appels de Suffixe_Cnjgtrs.
}
BEGIN { Trouver_Cnjgtrs }
{
   L'objet Conjugateurs est non-vide (C.Tete <> NIL)
}
   C.Conjugateur := C.Tete;
   WHILE C.Conjugateur^.Cle <> Cle DO BEGIN
{
 Q: Le conjugateur courant n'est pas le bon, mais il a un successeur
    (C.Conjugateur^.Suivant <> NIL)
}
      C.Conjugateur := C.Conjugateur^.Suivant;
   END; { WHILE }
{
   Le conjugateur courant est le bon
}
   C.Suffixe := C.Conjugateur^.Premier;
END; { Trouver_Cnjgtrs }

FUNCTION Suffixe_Cnjgtrs (VAR C       : TOCnjgtrs;
                          VAR Reponse : TChaine   ) : BOOLEAN;
{
   S'il reste des suffixes dans le conjugateur, rpond VRAI et
fournit le suffixe suivant dans Reponse; sinon, rpond FAUX.

Pr:  Le curseur du Cnjgtrs (C) a t amen en place par Trouver_Cnjgtrs et
      Suffixe_Cnjgtrs.

Post: S'il restait au moins un suffixe  lire,
      ALORS retourne ce suffixe dans Reponse et VRAI (et avance le curseur)
      SINON retourne FAUX.
}
BEGIN { Suffixe_Cnjgtrs }
   IF C.Suffixe = NIL
   THEN BEGIN
      Suffixe_Cnjgtrs := FALSE;
   END { THEN }
   ELSE BEGIN
      Suffixe_Cnjgtrs := TRUE;
      Reponse := C.Suffixe^.Suffixe;
      C.Suffixe := C.Suffixe^.Suivant;
   END; { IF }
END; { Suffixe_Cnjgtrs }

{ ******************** FIN DE L'OBJET CONJUGATEURS ******************** }
