{ ******************** DBUT DE L'OBJET DICO ******************** }
{
   Permet de charger, interroger, et dtruire un Dico.  Le Dico reprsente
des pellations (lettres minuscules seulement).  Lorsqu'on lui soumet un
"Mot" (chane de caractres), il peut rpondre:

Trouv		Si la chane fait partie des pellations apprises

Trop_Court	Si la chane est le prfixe d'une ou plusieurs pellations
		apprises

Trop_Long	Si un prfixe de la chane fait partie des pellations apprises

   Dans les trois cas, on retourne galement un entier donnant l'indice, dans le
Mot, de la premire lettre inconnue du Dico (ceci n'est utile que dans le cas
Trop_Long).

PRREQUIS:

   -CONST

   L'usager a dfini Max, qui donne la longueur maximale (moins un) du Dico, en
octets.

   -TYPES

   L'usager a dfini le type TNomFichier, une chane de caractres suffisante
pour recevoir un nom de fichier.  De mme pour le type TChaine, une chane de
caractres capable de recevoir un "mot" (longueur 31 devrait tre suffisante).

USAGE:

   CRATION/DESTRUCTION

F Lire_Dico (VAR Dico   : TDico;
	     Nom_Source : TNomFichier) : BOOLEAN

   Tent de crer en mmoire le Dico stock dans le fichier Nom_Source. 
Si le Dico est trop gros, retourne Faux.  Autrement, retourne Vrai.

P Detruire_Dico (VAR Dico : TDico)

   Dtruit le Dico et libre la mmoire.

   CONSULTATION

F Consulter_Dico (VAR Dico : TDico;
		  VAR Mot  : TChaine;
		  VAR I    : INTEGER) : TResultat;

   Dtermine si le Mot est dans le Dico (voir plus haut).

SIGN:

   Daniel U. Thibault     78 840 046
   13 Dcembre 1988
   26 Janvier 1989   Ajout des codes  et 
}
{ ++++++++++++++++++++ PROCDURES INVISIBLES ++++++++++++++++++++ }

PROCEDURE ObtMem (N     : UNSIGNED;
                  VAR P : [UNSAFE] UNSIGNED);
{
   Obtient un pointeur P  un bloc de N octets contigus.
}
   [EXTERNAL] PROCEDURE LIB$GET_VM (%REF N : UNSIGNED;
                                    %REF P : UNSIGNED);
   EXTERN;

BEGIN { ObtMem }
   LIB$GET_VM (N, P);
END; { ObtMem }

PROCEDURE LibMem (N     : UNSIGNED;
                  VAR P : [UNSAFE] UNSIGNED);
{
   Libre un bloc de N octets contigus dbutant  P.
}
   [EXTERNAL] PROCEDURE LIB$FREE_VM (%REF N : UNSIGNED;
                                     %REF P : UNSIGNED);
   EXTERN;

BEGIN { LibMem }
   LIB$FREE_VM (N, P);
END; { LibMem }

FUNCTION Coder (Lettre : CHAR) : INTEGER;
{
   Code la Lettre sur 6 bits (0..63).

Pr:  Lettre est un caractre.

Post: Retourne le code correspondant s'il existe (1..63), zro sinon.
}
BEGIN { Coder }
   CASE Lettre OF
      ' '      : Coder :=  1;
      '!'      : Coder :=  2;
      ''''     : Coder :=  3;
      '-'      : Coder :=  4;
      'A'      : Coder :=  5;
      'C'      : Coder :=  6;
      'D'      : Coder :=  7;
      'H'      : Coder :=  8;
      'J'      : Coder :=  9;
      'M'      : Coder := 10;
      'P'      : Coder := 11;
      'R'      : Coder := 12;
      'S'      : Coder := 13;
      'V'      : Coder := 14;
      'a'..'z' : Coder := ORD (Lettre) - 82;	{  97..122 -> 15..40 }
      ''      : Coder := 41;
      ''      : Coder := 42;
      ''      : Coder := 43;
      ''      : Coder := 44;
      ''      : Coder := 45;
      ''      : Coder := 46;
      ''      : Coder := 47;
      ''      : Coder := 48;
      ''      : Coder := 49;
      ''      : Coder := 50;
      ''      : Coder := 51;
      ''      : Coder := 52;
      ''      : Coder := 53;
      ''      : Coder := 54;
      ''      : Coder := 55;
      ''      : Coder := 56;
      ''      : Coder := 57;
      ''      : Coder := 58
   OTHERWISE
      Coder := 0
   END; { CASE }
END; { Coder }

FUNCTION Decoder_Lettre (VAR Dico : TDico;
                         Curseur  : INTEGER) : CHAR;
{
   Retourne le caractre cod par les 6 bits bas.

Pr:  Dico [Curseur] est un nud-lettre.

Post: Retourne la lettre code (CHR (0) si code illgal).
}
VAR
   Code : INTEGER;

BEGIN { Dcoder_Lettre }
   Code := ORD (Dico.Tete^[Curseur]) MOD 64;
   CASE Code OF
      1      : Decoder_Lettre :=  ' ';
      2      : Decoder_Lettre :=  '!';
      3      : Decoder_Lettre :=  '''';
      4      : Decoder_Lettre :=  '-';
      5      : Decoder_Lettre :=  'A';
      6      : Decoder_Lettre :=  'C';
      7      : Decoder_Lettre :=  'D';
      8      : Decoder_Lettre :=  'H';
      9      : Decoder_Lettre :=  'J';
     10      : Decoder_Lettre :=  'M';
     11      : Decoder_Lettre :=  'P';
     12      : Decoder_Lettre :=  'R';
     13      : Decoder_Lettre :=  'S';
     14      : Decoder_Lettre :=  'V';
     15..40  : Decoder_Lettre :=  CHR (Code + 82); { 15..40 -> a..z }
     41      : Decoder_Lettre :=  '';
     42      : Decoder_Lettre :=  '';
     43      : Decoder_Lettre :=  '';
     44      : Decoder_Lettre :=  '';
     45      : Decoder_Lettre :=  '';
     46      : Decoder_Lettre :=  '';
     47      : Decoder_Lettre :=  '';
     48      : Decoder_Lettre :=  '';
     49      : Decoder_Lettre :=  '';
     50      : Decoder_Lettre :=  '';
     51      : Decoder_Lettre :=  '';
     52      : Decoder_Lettre :=  '';
     53      : Decoder_Lettre :=  '';
     54      : Decoder_Lettre :=  '';
     55      : Decoder_Lettre :=  '';
     56      : Decoder_Lettre :=  '';
     57      : Decoder_Lettre :=  '';
     58      : Decoder_Lettre :=  ''
   OTHERWISE
     { 0, 59..63 }
      Decoder_Lettre := CHR (0)
   END; { CASE }
END; { Dcoder_Lettre }

FUNCTION Pas_de_Fils (Dico    : TDico;
                      Curseur : INTEGER) : BOOLEAN;
{
   Dtermine si le nud Curseur est sans fils ou non.

Pr:  Dico [Curseur] est un nud de type "Lettre".

Post: Retourne Vrai si le bit 7 est allum, Faux sinon.
      Curseur inchang.
}
BEGIN { Pas_de_Fils }
   Pas_de_Fils := (1 = (ORD (Dico.Tete^[Curseur]) DIV 128));
END; { Pas_de_Fils }

FUNCTION Fils_Suit (Dico    : TDico;
                    Curseur : INTEGER) : BOOLEAN;
{
   Dtermine si le nud Curseur compte le nud suivant parmi ses fils ou non.

Pr:  Dico [Curseur] est un nud de type "Lettre".

Post: Retourne Vrai si le bit 6 est allum, Faux sinon.
      Curseur inchang.
}
BEGIN { Fils_Suit }
   Fils_Suit := ODD (ORD (Dico.Tete^[Curseur]) DIV 64);
END; { Fils_Suit }

FUNCTION Est_une_feuille (Dico        : TDico;
                          VAR Curseur : INTEGER) : BOOLEAN;
{
   Dtermine si le nud Curseur est une feuille ou non.

Pr:  Dico [Curseur] est un nud de type "Lettre".

Post: Retourne Vrai si l'octet est nul, Faux sinon.
      Curseur avance de 2 si Vrai, 1 si Faux.
}
BEGIN { Est_une_Feuille }
   IF (0 = ORD (Dico.Tete^[Curseur + 1]))
   THEN BEGIN
      Est_une_Feuille := TRUE;
      Curseur := Curseur + 2;
   END { THEN }
   ELSE BEGIN
      Est_une_Feuille := FALSE;
      Curseur := Curseur + 1;
   END; { IF }
END; { Est_une_Feuille }

FUNCTION Lire_Decalage (Dico        : TDico;
                        VAR Curseur : INTEGER;
                        VAR Dernier : BOOLEAN) : INTEGER;
{
   Lit le dcalage cod par Curseur.

Pr:  Dico [Curseur] est un nud de type "Dcalage" (le premier octet d'un
      pointeur complet).

Post: Retourne la valeur du dcalage.  Dernier est retourn Vrai si le pointeur
      tait le dernier fils du nud.
      Curseur est avanc  l'octet suivant le dernier octet faisant partie
      du pointeur.
}
VAR
   Dec : INTEGER;

BEGIN { Lire_Dcalage }
   Dec := 0;
   WHILE ORD (Dico.Tete^[Curseur]) >= 128 DO BEGIN
      Dec := (Dec*128) + (ORD (Dico.Tete^[Curseur]) MOD 128);
      Curseur := Curseur + 1;
   END; { WHILE }
   Dernier := (ORD (Dico.Tete^[Curseur]) >= 64);
   Dec     := (Dec*64) + (ORD (Dico.Tete^[Curseur]) MOD 64);
   Curseur := Curseur + 1;
   Lire_Decalage := Dec;
END; { Lire_Dcalage }

FUNCTION Lire_Lettre_Codee (Dico    : TDico;
                            Curseur : INTEGER) : INTEGER;
{
   Lit la lettre (encore en code) du nud Curseur.

Pr:  Dico [Curseur] est un nud de type "Lettre".

Post: Retourne les 6 bits infrieurs de Dico [Curseur].
      Curseur inchang.
}
BEGIN { Lire_Lettre_Code }
   Lire_Lettre_Codee := ORD (Dico.Tete^[Curseur]) MOD 64;
END; { Lire_Lettre_Code }

{ ++++++++++++++++++++ PROCDURES VISIBLES ++++++++++++++++++++ }

{ %%% CRATION/DESTRUCTION %%% }

FUNCTION Lire_Dico (VAR Dico   : TDico;
		    Nom_Source : TNomFichier) : BOOLEAN;
{
   Tente de lire, du fichier Nom_Source, le Dico.

Pr:  Nom_Source est un fichier contenant le Dico.

Post: S'il n'est pas trop gros,
      ALORS Dico est en mmoire, prt  tre consult, et VRAI est retourn
      SINON Faux est retourn
}
VAR
   I,
   Curseur : INTEGER;	{ Ira de 0  Dico.Longueur - 1 }
   Octet   : BYTE;
   Source  : FILE OF BYTE;

BEGIN { Lire_Dico }
   OPEN (Source, Nom_Source, OLD);
   RESET (Source);
{
   La longueur du Dico.
}
   FOR I := 0 TO 3 DO READ (Source, Dico.Octets [I]);
   IF Dico.Longueur <= (Max + 1)
   THEN BEGIN
{
   Rserver la mmoire en un seul bloc continu.
}
      ObtMem (Dico.Longueur, Dico.Tete);

      Curseur := 0;
      WHILE NOT EOF (Source) DO BEGIN
         READ (Source, Dico.Tete^[Curseur]);
         Curseur := Curseur + 1;
      END; { WHILE }
   Lire_Dico := TRUE;
   END { THEN }
   ELSE BEGIN
      Lire_Dico := FALSE;
   END; { IF }
   CLOSE (Source);
END; { Lire_Dico }

PROCEDURE Detruire_Dico (VAR Dico : TDico);
{
   Dtruit le Dico.

Pr:  Dico existe.

Post: La mmoire est libre
}
BEGIN { Dtruire_Dico }
   LibMem (Dico.Longueur, Dico.Tete);
   Dico.Tete := NIL;
   Dico.Longueur := 0;
END; { Dtruire_Dico }

{ %%% CONSULTATION %%% }

FUNCTION Consulter_Dico (VAR Dico : TDico;
                         VAR Mot  : TChaine;
			 VAR I    : INTEGER) : TResultat;
{
   Consulte le Dico  propos du mot Mot.

Pr:  Dico a t lu.  Mot est une chane quelconque.

Post:	Si l'pellation apparat dans le Dico,
	   Rpond Trouve, et I = 1 + Longueur (Mot)
	Si l'pellation apparat en prfixe dans le Dico,
	   Rpond Trop_Court, et I = 1 + Longueur (Mot)
	Si l'pellation apparat en partie dans le Dico,
	   Rpond Trop_Long, et
	   I = 1 + Longueur (partie du Mot apparaissant dans le Dioc)
}
VAR
   Decalage,
   Curseur   : INTEGER;
   Resultat  : TResultat;
   Suivant,
   Feuille,
   Sans_Fils,
   Dernier,
   OK        : BOOLEAN;

BEGIN { Consulter_Dico }
   Curseur  := 0;
   I        := 1;
   Resultat := Indetermine;
   WHILE Resultat = Indetermine DO BEGIN
{
   Q : Mot [I..] reste  trouver
       Mot [1..I[ correspond au chemin menant  Dico [Curseur]
       Dico [Curseur] est un nud Lettre.
}
      Sans_Fils := Pas_de_Fils (Dico, Curseur);
      Suivant   := Fils_Suit (Dico, Curseur);
      Feuille   := Est_une_Feuille (Dico, Curseur);
      IF I > LENGTH (Mot)
      THEN BEGIN
{
   On a trouv toutes les lettres
}
         IF Feuille
         THEN BEGIN
            Resultat := Trouve;
         END { THEN }
         ELSE BEGIN
            Resultat := Trop_Court;
         END; { IF }
      END { THEN }
      ELSE BEGIN
{
   Il reste des lettres  trouver: chercher parmi les fils du nud courant
}
         IF Sans_Fils
         THEN BEGIN
{
   Le nud n'a d'autres fils que la Queue (non pertinente)
   et (peut-tre) le nud suivant
}
            IF Suivant
            THEN BEGIN
{
   Le nud suivant est un fils (et on y est rendu)
}
               IF Lire_Lettre_Codee (Dico, Curseur) = Coder (Mot [I])
               THEN BEGIN
                  I := I + 1;
               END { THEN }
               ELSE BEGIN
                  Resultat := Trop_Long;
               END; { IF }
            END { THEN }
            ELSE BEGIN
{
   Le nud suivant n'tant pas un fils, on a puis tous les recours
}
               Resultat := Trop_Long;
            END; { IF }
         END { THEN }
         ELSE BEGIN
{
   Le nud a des fils loigns
}
            OK := FALSE;
            Decalage := 0;
            REPEAT
               Decalage := Decalage + Lire_Decalage (Dico, Curseur, Dernier);
               IF Lire_Lettre_Codee (Dico, Curseur + Decalage) = Coder (Mot [I])
               THEN BEGIN
                  I := I + 1;
                  Curseur := Curseur + Decalage;
                  OK := TRUE;
               END; { IF }
            UNTIL Dernier OR OK;
            IF NOT OK
            THEN BEGIN
{
   La boucle s'est termine sans trouver la lettre parmi les fils loigns;
   reste peut-tre le nud suivant (auquel on est rendu)
}
               IF Suivant
               THEN BEGIN
                  IF Lire_Lettre_Codee (Dico, Curseur) = Coder (Mot [I])
                  THEN BEGIN
                     I := I + 1;
                  END { THEN }
                  ELSE BEGIN
                     Resultat := Trop_Long;
                  END; { IF }
               END { THEN }
               ELSE BEGIN
                  Resultat := Trop_Long;
               END; { IF }
            END; { IF }
         END; { IF }
      END; { IF }
   END; { WHILE }
   Consulter_Dico := Resultat;
END; { Consulter_Dico }

{ ******************** FIN DE L'OBJET DICO ******************** }
