PROGRAM Enumerer (INPUT, OUTPUT);
{
   Daniel U. Thibault  78 840 046
   26..31 Janvier 1989

   numre les pellations lgales d'un Dico (recherche de faux mots).
}
CONST
   Max         = 524287; { 512k - 1 }

TYPE
   TChaine         = VARYING [80] OF CHAR;
   TNomFichier     = TChaine;
   T64             = 1..64;
   TTab64          = ARRAY [T64] OF INTEGER;

%INCLUDE 'DICO.TYP'

VAR
   Nom  : TChaine;
   Dico : TDico;

%INCLUDE 'DICO.PAS'

FUNCTION Plus_Grand (VAR Dico : TDico;
                     A, B     : INTEGER) : BOOLEAN;
{
   Dtermine si la lettre code par Dico [A] est > que celle de Dico [B],
au sens lexicographique.
}
VAR
   Lex    : VARYING [64] OF CHAR;
   CA, CB : CHAR;
   IA, IB : INTEGER;

BEGIN { Plus_Grand }
   CA := Decoder_Lettre (Dico, A);
   CB := Decoder_Lettre (Dico, B);
   Lex := ' !''-AabCcDdefgHhiJjklMmnoPpqRrSstuVvwxyz';
   IA := 1;
   WHILE Lex [IA] <> CA DO IA := IA + 1;
   IB := 1;
   WHILE Lex [IB] <> CB DO IB := IB + 1;
   Plus_Grand := (IA > IB);
END; { Plus_Grand }

PROCEDURE Tri_Fils (VAR Dico : TDico;
                    VAR Fils : TTab64;
                    NFils    : T64);
{
   Trie le tableau des pointeurs fils en ordre lexicographique croissant.
(selon tri bulle car le tableau compte rarement plus de 3 ou 4 lments).

Pr:  Fils [1..NFils] existe.

Post: Fils [1..NFils] est tri en ordre lexicographique croissant.
}
VAR
   I, J, L : T64;
   Dec     : INTEGER;

BEGIN { Tri_Fils }
   L := NFils;
   WHILE L > 1 DO BEGIN
{
   Q : Fils [1..L] reste  trier.  1 <= L <= NFils.
}
      J := 1;
      I := 1;
      WHILE J < L DO BEGIN
         J := J + 1;
         IF Plus_Grand (Dico, Fils [J], Fils [I]) THEN I := J;
{
      Q : 1 <= I <= J <= L et
          Fils [I] est le plus grand terme de l'intervalle [1..J]
}
      END; { WHILE }
      IF I < L
      THEN BEGIN
{
   Fils [I, L] := Fils [L, I]
}
         Dec      := Fils [I];
         Fils [I] := Fils [L];
         Fils [L] := Dec;
      END; { IF }
      L := L - 1;
   END; { WHILE }
END; { Tri_Fils }

PROCEDURE Enumerer_Mots (VAR Dico : TDico;
                         Curseur  : INTEGER;
                         Mot      : TChaine);
{
   Trouve les mots ayant Mot comme prfixe.
}
VAR
   C         : CHAR;
   Dec,
   Decalage  : INTEGER;
   Suivant,
   Feuille,
   Sans_Fils,
   Dernier   : BOOLEAN;
   I,
   NFils     : T64;
   Fils      : TTab64;

BEGIN { numrer_Mots }
   IF Curseur < Dico.Longueur
   THEN BEGIN
{
   Dico [Curseur] est un nud Lettre.
}
      IF Curseur <> 0 THEN Mot := Mot + Decoder_Lettre (Dico, Curseur);
      Sans_Fils := Pas_de_Fils (Dico, Curseur);
      Suivant   := Fils_Suit (Dico, Curseur);
      Feuille   := Est_une_Feuille (Dico, Curseur);

      IF Feuille THEN WRITELN (Mot);

      NFils := 0;
      IF NOT Sans_Fils
      THEN BEGIN
{
   Le nud a des fils loigns
}
         Decalage := 0;
         REPEAT
            Dec := Lire_Decalage (Dico, Curseur, Dernier);
            Decalage := Decalage + Dec;
            NFils := NFils + 1;
            Fils [NFils] := Curseur + Decalage;
         UNTIL Dernier;
      END; { IF }

      IF Suivant
      THEN BEGIN
            NFils := NFils + 1;
            Fils [NFils] := Curseur;
      END; { IF }

      Tri_Fils (Dico, Fils, NFils);

      FOR I := 1 TO NFils DO BEGIN
         Enumerer_Mots (Dico, Fils [I], Mot);
      END; { FOR }
   END; { IF }
END; { numrer_Mots }
{
 *************************************************************************
}
BEGIN { numrer }
   WRITELN ('Programme numrant les pellations contenues dans un .DIC');
   WRITELN ('Urhixidur, Janvier 1989');
   WRITELN;
   WRITE ('Fichier o se trouve le Dico? ');
   READLN (Nom);
   WRITELN; WRITELN;

   IF NOT Lire_Dico (Dico, Nom) THEN HALT;

   Enumerer_Mots (Dico, 0, '');

   WRITELN;
   WRITELN ('numration termine');
   WRITELN;

   Detruire_Dico (Dico);
END. { numrer }
