{
 ******************** DBUT DE L'OBJET LISTE ********************
}
{
   L'objet Liste permet de manipuler des listes d'lments de type TContListe.
Une Liste peut tre vide ou compter un certain nombre d'lments.  Les lments
membres d'une Liste sont supposs uniques.

PRREQUIS:

   -AUTRES OBJETS

   L'objet Mmoire doit exister.

   -TYPE

   Le type TContListe a t dfini.  C'est un type ordinal simple (il peut
subir les oprations de comparaison (= et <>) et tre retourn par une
Fonction).

USAGE:

   globales:

P Creer_Liste (VAR Liste : TListe)		   Cre une Liste vide.

P Detruire_Liste (VAR Liste : TListe)		   Vide une Liste.

   ajout/retrait d'lments:

P Ajouter_Tete_Liste (VAR Liste : TListe;	   Ajoute un contenu 
		      C         : TContListe)      la tte de la Liste,
						   sans vrification.

P Enlever_Liste (VAR Liste : TListe;		   Enlve un contenu de
                 C         : TContListe)	   la Liste.

   diagnostic:

F Vide_Liste (Liste : TListe) : BOOLEAN		Dtermine si la Liste est vide
						ou non.

F Taille_Liste (Liste : TListe) : INTEGER	Dtermine le nombre d'lments
						membres de la Liste.

   manipulation du curseur:

P Debut_Liste (Liste       : TListe;		Place le Curseur au dbut
               VAR Curseur : TCurseurListe)	de la Liste.

F Suivant_Liste (VAR Curseur : TCurseurListe) : BOOLEAN	Tente d'avancer le
							Curseur au prochain
							lment de la Liste.

   consultation:

F Lire_Liste (Curseur : TCurseurListe) : TContListe	Lit le contenu courant.

F Dans_Liste (Liste : TListe;			Dtermine si C apparat
              C     : TContListe) : BOOLEAN	dans la Liste ou non.

F Comparer_Listes (Liste1, Liste2 : TListe) : BOOLEAN	Compare deux listes,
							retourne Vrai si elles
							ont les mmes contenus.

NOTES:

   L'unicit des membres d'une mme Liste est une responsabilit qui incombe 
l'usager.  Elle est ncessaire pour que Comparer_Liste et Enlever_Liste
marchent correctement.

   La liste est ralise par une liste chane de nuds, chaque nud ayant un
Contenu et un successeur.  La variable Liste elle-mme pointe  la tte de la
Liste.

SIGN:

Daniel U. Thibault     78 840 046

28 Novembre 1988
}

{ ++++++++++++++++++++ PROCDURES VISIBLES ++++++++++++++++++++ }

{ %%% GNRALES %%% }

PROCEDURE Creer_Liste (VAR Liste : TListe);
{
   Cre une Liste vide.

Pr:  Liste ne doit pas tre une liste active
      (sinon l'usager perdra l'accs  cette mmoire).

Post: Liste = NIL
}
BEGIN { Crer_Liste }
   Liste := NIL
END; { Crer_Liste }

PROCEDURE Detruire_Liste (VAR Liste : TListe);
{
   Vide une Liste.

Pr:  Aucun  moins que la Liste ne contienne des pointeurs; auquel cas la
      responsabilit des zones mortes (si applicable) incombe  l'usager.

Post: La Liste est vide.
}
VAR
   Curseur : TCurseurListe;

BEGIN { Dtruire_Liste }
   WHILE Liste <> NIL DO BEGIN
      Curseur := Liste;
      Liste := Liste^.Suivant;
      Lib_Noeud_Mem (Curseur);
   END { WHILE }
END; { Dtruire_Liste }

{ %%% AJOUT/RETRAIT D'LMENTS %%% }

PROCEDURE Ajouter_Tete_Liste (VAR Liste : TListe;
                              C         : TContListe);
{
   Ajoute un Contenu  la tte de la Liste, sans vrifier si C tait dj
membre de la Liste ou non.  L'USAGER EST RESPONSABLE DU MAINTIEN DE L'UNICIT
DES LMENTS DE LISTE.

Pr:  La Liste existe.

Post: C apparat en tte de la Liste.
}
VAR
   Curseur : TCurseurListe;

BEGIN { Ajouter_Tte_Liste }
   Obt_Noeud_Mem (Curseur);
   Curseur^.Courant := C;
   Curseur^.Suivant := Liste;
   Liste := Curseur;
END; { Ajouter_Tte_Liste }

PROCEDURE Enlever_Liste (VAR Liste : TListe;
                         C         : TContListe);
{
   Enlve C de la Liste.  Il n'est pas ncessaire que C soit membre avant
l'opration.

Pr:  La Liste existe.

Post: C n'est plus un lment de la Liste.
}
VAR
   Precedent,
   Curseur    : TCurseurListe;

BEGIN { Enlever_Liste }
   IF Liste <> NIL
   THEN BEGIN
      IF Liste^.Courant = C
      THEN BEGIN
{
   Le nud  enlever est en tte
}
         Curseur := Liste;
         Liste   := Curseur^.Suivant;
         Lib_Noeud_Mem (Curseur);
      END { THEN }
      ELSE BEGIN
{
   Le nud  enlever n'est pas en tte
}
         Precedent := Liste;
         Curseur   := Liste^.Suivant;
         WHILE Curseur <> NIL DO BEGIN
{
   Q : Precedent est le pre de Curseur
       Les prdcesseurs de Curseur sont diffrents de C
}
            IF C <> Curseur^.Courant
            THEN BEGIN
               Precedent := Curseur;
               Curseur := Curseur^.Suivant;
            END { THEN }
            ELSE BEGIN
               Precedent^.Suivant := Curseur^.Suivant;
               Lib_Noeud_Mem (Curseur);
            END; { IF }
         END; { WHILE }
      END; { IF }
   END { IF }
END; { Enlever_Liste }

{ %%% DIAGNOSTIC %%% }

FUNCTION Vide_Liste (Liste : TListe) : BOOLEAN;
{
   Dtermine si la Liste est vide ou non.

Pr:  La Liste existe.

Post: Retourne Vrai si la Liste est vide, Faux sinon.
}
BEGIN { Vide_Liste }
   Vide_Liste := (Liste = NIL)
END; { Vide_Liste }

FUNCTION Taille_Liste (Liste : TListe) : INTEGER;
{
   Retourne le nombre d'lments dans la Liste.

Pr:  La Liste existe.

Post: Retourne le nombre d'lments dans la Liste.
}
VAR
   Nombre  : INTEGER;
   Curseur : TCurseurListe;

BEGIN { Taille_Liste }
   Nombre := 0;
   Curseur := Liste;
   WHILE Curseur <> NIL DO BEGIN
      Nombre := Nombre + 1;
      Curseur := Curseur^.Suivant;
   END; { WHILE }
   Taille_Liste := Nombre
END; { Taille_Liste }

{ %%% MANIPULATION DU CURSEUR %%% }

PROCEDURE Debut_Liste (Liste       : TListe;
                       VAR Curseur : TCurseurListe);
{
   Amne le Curseur au dbut de la Liste.

Pr:  La Liste existe (pourrait tre vide).

Post: Le Curseur est au dbut de la Liste.
}
BEGIN { Dbut_Liste }
   Curseur := Liste
END; { Dbut_Liste }

FUNCTION Suivant_Liste (VAR Curseur : TCurseurListe) : BOOLEAN;
{
   Tente d'avancer le Curseur au prochain lment de la Liste.

Pr:  Le Curseur est valide (non-NIL, pointe  un TNoeudListe).

Post: Retourne Faux si l'lment de la Liste n'a pas de successeur
      (et retourne un Curseur NIL);
      sinon retourne Vrai et avance le Curseur  l'lment suivant.

   NB: Remarquez que la variable Liste n'est pas ncessaire.
}
BEGIN { Suivant_Liste }
   Curseur := Curseur^.Suivant;
   Suivant_Liste := (Curseur <> NIL)
END; { Suivant_Liste }

{ %%% CONSULTATION %%% }

FUNCTION Lire_Liste (Curseur : TCurseurListe) : TContListe;
{
   Lit le Contenu courant de la Liste.

Pr:  Le Curseur est valide (non-NIL, pointe  un TNoeudListe).

Post: Retourne le contenu de l'lment courant (Curseur) de la Liste.

   NB: Remarquez que la variable Liste n'est pas ncessaire.
}
BEGIN { Lire_Liste }
   Lire_Liste := Curseur^.Courant
END; { Lire_Liste }

FUNCTION Dans_Liste (Liste : TListe;
                     C     : TContListe) : BOOLEAN;
{
   Dtermine si C apparat dans la Liste ou non.

Pr:  La Liste existe.

Post: Retourne Vrai ssi l'lment C appartient  la Liste.
}
VAR
   Curseur : TCurseurListe;

BEGIN { Dans_Liste }
   Dans_Liste := FALSE;
   Curseur := Liste;
   WHILE Curseur <> NIL DO BEGIN
{
   Q : Les prdcesseurs de Curseur avaient tous un Contenu diffrent de C
}
      IF Curseur^.Courant = C
      THEN BEGIN
         Curseur := NIL;
         Dans_Liste := TRUE;
      END { THEN }
      ELSE BEGIN
         Curseur := Curseur^.Suivant;
      END; { IF }
   END { WHILE }
END; { Dans_Liste }

FUNCTION Comparer_Listes (Liste1, Liste2 : TListe) : BOOLEAN;
{
   Compare deux listes, retourne Vrai si elles ont les mmes contenus.

Pr:  Les Listes existent

Post: Retourne Vrai ssi chacun des lments de l'une apparat dans l'autre.
}
VAR
   Curseur : TCurseurListe;

BEGIN { Comparer_Listes }
   IF Taille_Liste (Liste1) = Taille_Liste (Liste2)
   THEN BEGIN
      Comparer_Listes := TRUE;
      Curseur := Liste1;
      WHILE Curseur <> NIL DO BEGIN
{
   Q : Les prdcesseurs de Curseur (dans Liste1) avaient tous des Contenus
       apparaissant dans Liste2.
}
         IF Dans_Liste (Liste2, Curseur^.Courant)
         THEN BEGIN
            Curseur := Curseur^.Suivant;
         END { THEN }
         ELSE BEGIN
            Curseur := NIL;
            Comparer_Listes := FALSE;
         END; { IF }
      END; { WHILE }
   END { THEN }
   ELSE BEGIN
{
   Tailles diffrentes: identit impossible
}
      Comparer_Listes := FALSE;
   END { IF }
END; { Comparer_Listes }
