{
******************** DBUT DE L'OBJET MMOIRE ********************
}
{
   L'objet mmoire permet une gestion plus efficace de la mmoire du VAX.

PRREQUIS:

   -CONST

   L'usager a dfini la taille de ses atomes de mmoire (en octets) avec la
constante entire SNoeud.  SNoeud doit tre plus grand ou gal  8.

   -VAR

   La variable globale Mem (de type TMemoire) a t cre.

USAGE:

P Init_Mem				Initialise l'objet
P Obt_Noeud_Mem (VAR P : Pointeur)	Obtient un pointeur P  un nud
P Lib_Noeud_Mem (VAR P : Pointeur)	Libre la mmoire pointe par P

NOTES:

   Le type Pointeur est quelconque et peut changer d'un appel  l'autre car il
est UNSAFE.

   Si le systme refuse d'accorder la mmoire, l'objet plante.  C'est 
l'usager de s'assurer de ses quotas avant de lancer l'excution.

   La mmoire est obtenue du systme en blocs de 1024*SNoeud octets (afin de
minimiser les frais gnraux du systme).  La variable Mem pointe en tout temps
au premier bloc de mmoire disponible.  Chaque bloc contient sa propre
description: longueur (en octets) et un pointeur de chanage avant vers son
successeur.  La mmoire alloue  l'usager provient de la tte de la liste.  La
mmoire retourne par l'usager est simplement mise en tte de liste,  la faon
d'une pile.

   AMLIORATIONS POSSIBLES:  Collecte des ordures (maintien de la liste en
ordre et retour des blocs au systme lorsque possible).

SIGN:

Daniel U. Thibault     78 840 046

28 Novembre 1988
}

{ ++++++++++++++++++++ PROCDURES INVISIBLES ++++++++++++++++++++ }

PROCEDURE ObtMem (N     : UNSIGNED;
                  VAR P : [UNSAFE] UNSIGNED);
{
   Obtient un pointeur P  un bloc de N octets contigus.
   Pas accessible  l'usager.
}
   [EXTERNAL] PROCEDURE LIB$GET_VM (%REF N : UNSIGNED;
                                    %REF P : UNSIGNED);
   EXTERN;

BEGIN { ObtMem }
   LIB$GET_VM (N, P);
END; { ObtMem }

PROCEDURE LibMem (N     : UNSIGNED;
                  VAR P : [UNSAFE] UNSIGNED);
{
   Libre un bloc de N octets contigus dbutant  P.
   Pas accessible  l'usager.
}
   [EXTERNAL] PROCEDURE LIB$FREE_VM (%REF N : UNSIGNED;
                                     %REF P : UNSIGNED);
   EXTERN;

BEGIN { LibMem }
   LIB$FREE_VM (N, P);
END; { LibMem }

{ ++++++++++++++++++++ PROCDURES VISIBLES ++++++++++++++++++++ }

PROCEDURE Init_Mem;
{
   Initialise la mmoire.

Pr:  Mem (variable globale) n'est pas dj en usage
      (autrement l'usager perd l'accs  la mmoire dj rserve).

Post: Mem est utilisable.
}
BEGIN { Init_Mem }
   Mem.Pntr := NIL;
END; { Init_Mem }

PROCEDURE Obt_Noeud_Mem (VAR P : [UNSAFE] UNSIGNED);
{
   Obtient un pointeur P  un nud en mmoire (de taille SNoeud octets).

Pr:  Mem (variable globale) est utilisable et
      le systme ne refusera pas l'allocation (si ncessaire).

Post: P est dfini et pointe  un bloc de mmoire de grandeur SNoeud.
}
VAR
   NMem : TMemoire;

BEGIN { Obt_Nud_Mem }
{
   Obtenir un autre bloc s'il n'en reste plus de disponible
}
   IF Mem.Pntr = NIL
   THEN BEGIN
      ObtMem (SNoeud*1024, Mem.Pntr);
      Mem.Pntr^.Longueur := SNoeud*1024;
      Mem.Pntr^.Suivant  := NIL;
   END; { IF }
{
   Mem.Pntr pointe au premier bloc disponible
}
   IF Mem.Pntr^.Longueur = SNoeud
   THEN BEGIN
{
   Le bloc va disparatre une fois l'allocation termine
}
      NMem.Pntr := Mem.Pntr^.Suivant;
   END { THEN }
   ELSE BEGIN
{
   Le bloc se raccourcit
}
      NMem.Entr := Mem.Entr + SNoeud;
      NMem.Pntr^.Longueur := Mem.Pntr^.Longueur - SNoeud;
      NMem.Pntr^.Suivant  := Mem.Pntr^.Suivant;
   END; { IF }
   P := Mem.Entr;
   Mem.Pntr := NMem.Pntr;
END; { Obt_Nud_Mem }

PROCEDURE Lib_Noeud_Mem (VAR P : [UNSAFE] UNSIGNED);
{
   Libre la mmoire (de taille SNoeud) associe au pointeur P.

Pr:  Mem (variable globale) est utilisable, et
      P pointe  un bloc de taille SNoeud.

Post: P est NIL.
}
VAR
   NMem : TMemoire;

BEGIN { Lib_Nud_Mem }
   NMem.Entr := P;
   NMem.Pntr^.Longueur := SNoeud;
   NMem.Pntr^.Suivant  := Mem.Pntr;
   Mem.Pntr := NMem.Pntr;
   P := 0;
END; { Lib_Nud_Mem }
