{
 ******************** DBUT DE L'OBJET TAMIS ********************
}
{
   Le Tamis est une suite ordonne de listes.  Les listes sont classes
en ordre dcroissant grce  un Poids.  On peut "dpiler" un lment du Tamis:
l'lment proviendra de la Liste couramment en tte du Tamis.  De plus, on peut
parcourir le Tamis, dans l'ordre de dpilement, sans le vider.

   Le Tamis sert donc (a)  grer une file d'attente avec priorit, ou
(b)  effectuer un tri par tas.

PRREQUIS:

   Les poids doivent tre dans l'intervalle 0..255 (type BYTE).
   Les objets Liste et Memoire doivent exister.

USAGE:

   gnrales:

P Creer_Tamis (VAR Tamis : TTamis)		Cre un Tamis vide.

P Detruire_Tamis (VAR Tamis : TTamis)		Dtruit un Tamis.

   ajout/retrait d'lments:

P Ajouter_Tamis (VAR Tamis : TTamis,		Ajoute un contenu C  la liste
		 C	   : TContListe,	du Tamis de poids Poids.
		 Poids	   : BYTE)

P Enlever_Tamis (VAR Tamis : TTamis,		Enlve C de la liste de poids
		 C	   : TContListe,	Poids du Tamis.
		 Poids	   : BYTE)

   consultation destructive:

F Depiler_Tamis (VAR Tamis : TTamis,		Tente de dpiler un contenu C
		 VAR C	   : TContListe)	de la premire liste du Tamis.

   consultation non-destructive:

F Debut_Tamis (Tamis	: TTamis,		Tente de placer un Curseur au
	       VAR Curs : TCurseurTamis)	dbut du Tamis.

F Suivant_Tamis (VAR Curs : TCurseurTamis)	Tente d'avancer le Curseur 
						la Liste suivante du Tamis.

NOTES:

   Dans le cas d'une consultation non-destructive, l'algorithme est comme suit:

IF Debut_Tamis (Tamis, Curseur_Tamis)
THEN BEGIN
   REPEAT
      REPEAT
         Curseur_Tamis.Curseur pointe  l'lment du Tamis qui nous intresse
      UNTIL NOT Suivant_Liste (Curseur_Tamis.Curseur);
   UNTIL NOT Suivant_Tamis (Curseur_Tamis);
END; 

   On pourrait complter l'encapsulation en introduisant les procdures:

F Suivant_E_Tamis (VAR Curs : TCurseurTamis)	Tente d'avancer le Curs 
						l'lment suivant du Tamis.

F Lire_E_Tamis (Curs : TCurseurTamis)		Lit l'lment courant du Tamis

   L'algorithme de consultation non-destructive deviendrait:

IF Debut_Tamis (Tamis, Curseur_Tamis)
THEN BEGIN
   REPEAT
      REPEAT
         Lire_E_Tamis (Curseur_Tamis)
         nous retourne l'lment qui nous intresse
      UNTIL NOT Suivant_E_Tamis (Curseur_Tamis);
   UNTIL NOT Suivant_Tamis (Curseur_Tamis);
END;

SIGN:

   Daniel U. Thibault     78 840 046

   29 Novembre 1988
}

{ ++++++++++++++++++++ PROCDURES VISIBLES ++++++++++++++++++++ }

{ %%% GNRALES %%% }

PROCEDURE Creer_Tamis (VAR Tamis : TTamis);
{
   Cre un Tamis vide.

Pr:  Le Tamis n'existe pas encore (ou est vide).
      Sinon, l'usager perdra l'accs  une partie de la mmoire.

Post: Tamis = NIL.
}
BEGIN { Crer_Tamis }
   Tamis := NIL;
END; { Crer_Tamis }

PROCEDURE Detruire_Tamis (VAR Tamis : TTamis);
{
   Dtruit un Tamis.

Pr:  Tamis existe.

Post: Tamis est vide (= NIL).
}
VAR
   P : TPTeteListe;

BEGIN { Dtruire_Tamis }
   WHILE Tamis <> NIL DO BEGIN
{
   Q : Le Tamis compte au moins une Liste membre.
}
      Detruire_Liste (Tamis^.Liste);
      P := Tamis;
      Tamis := Tamis^.Suivant;
      Lib_Noeud_Mem (P);
   END; { WHILE }
END; { Dtruire_Tamis }

{ %%% AJOUT/RETRAIT D'LMENTS %%% }

PROCEDURE Ajouter_Tamis (VAR Tamis : TTamis;
                         C         : TContListe;
                         Poids     : BYTE);
{
   Ajoute C  la Liste de poids Poids du Tamis.

Pr:  Tamis existe.

Post: La Liste de poids Poids existe, et C en est membre.
}
VAR
   P, Curseur : TPTeteListe;
   Resolu     : BOOLEAN;

BEGIN { Ajouter_Tamis }
   IF Tamis = NIL
   THEN BEGIN
{
   Le Tamis tait vide: crer une liste d'un lment
}
      Obt_Noeud_Mem (Tamis);
      Tamis^.Poids   := Poids;
      Tamis^.Suivant := NIL;
      Creer_Liste (Tamis^.Liste);
      Ajouter_Tete_Liste (Tamis^.Liste, C);
   END { THEN }
   ELSE BEGIN
{
   Le Tamis compte dj des listes membres
}
      Curseur := Tamis;
      Resolu := FALSE;
      WHILE NOT Resolu DO BEGIN
         IF Curseur^.Poids > Poids
         THEN BEGIN
{
   La liste recherche est parmi les successeurs de la liste courante
}
            IF Curseur^.Suivant <> NIL
            THEN BEGIN
               Curseur := Curseur^.Suivant;
            END { THEN }
            ELSE BEGIN
{
   La liste recherche n'existait pas: la crer en queue de Tamis
}
               Obt_Noeud_Mem (Curseur^.Suivant);
               Curseur := Curseur^.Suivant;
               Curseur^.Poids   := Poids;
               Curseur^.Suivant := NIL;
               Creer_Liste (Curseur^.Liste);
               Ajouter_Tete_Liste (Curseur^.Liste, C);
               Resolu := TRUE;
            END; { IF }
         END { THEN }
         ELSE IF Curseur^.Poids = Poids
         THEN BEGIN
{
   On a trouv la liste recherche: y ajouter l'lment
}
            Ajouter_Tete_Liste (Curseur^.Liste, C);
            Resolu := TRUE;
         END { THEN }
         ELSE BEGIN { Curseur^.Poids < Poids }
{
   La liste recherche n'existait pas, et doit prcder la liste courante
}
            Obt_Noeud_Mem (P);
            P^ := Curseur^;
            Curseur^.Poids := Poids;
            Curseur^.Suivant := P;
            Creer_Liste (Curseur^.Liste);
            Ajouter_Tete_Liste (Curseur^.Liste, C);
            Resolu := TRUE;
         END; { ELSE-IF }
      END; { WHILE }
   END; { IF }
END; { Ajouter_Tamis }

PROCEDURE Enlever_Tamis (VAR Tamis : TTamis;
                         C         : TContListe;
                         Poids     : BYTE);
{
   Enlve le contenu C du Tamis.

Pr:  Tamis existe,
      C est unique dans la Liste de poids Poids (ou n'existe pas).

Post: C n'apparat plus dans la Liste de poids Poids du Tamis.
}
VAR
   Curseur : TPTeteListe;
   Resolu  : BOOLEAN;

BEGIN { Enlever_Tamis }
   IF Tamis <> NIL
   THEN BEGIN
{
   Le Tamis n'est pas vide: trouver la bonne liste
}
      Curseur := Tamis;
      Resolu := FALSE;
      WHILE NOT Resolu DO BEGIN
         IF Curseur^.Poids > Poids
         THEN BEGIN
{
   La liste recherche est parmi les successeurs de la liste courante
}
            IF Curseur^.Suivant <> NIL
            THEN BEGIN
               Curseur := Curseur^.Suivant;
            END { THEN }
            ELSE BEGIN
{
   La liste recherche n'appartenait pas au Tamis: mission accomplie
}
               Resolu := TRUE;
            END; { IF }
         END { THEN }
         ELSE IF Curseur^.Poids = Poids
         THEN BEGIN
{
   On a trouv la Liste: en enlever le contenu C
}
            Enlever_Liste (Curseur^.Liste, C);
            Resolu := TRUE;
         END { THEN }
         ELSE BEGIN { Curseur^.Poids < Poids }
{
   La liste recherche n'appartenait pas au Tamis: mission accomplie
}
            Resolu := TRUE;
         END; { ELSE-IF }
      END; { WHILE }
   END; { IF }
END; { Enlever_Tamis }

{ %%% CONSULTATION DESTRUCTIVE %%% }

FUNCTION Depiler_Tamis (VAR Tamis : TTamis;
                        VAR C     : TContListe) : BOOLEAN;
{
   Dpile le premier contenu du Tamis (ie un lment membre du Tamis, tel
qu'il n'existe pas d'lment de Poids plus important).

Pr:  Tamis existe.

Post: Retourne Vrai si le Tamis tait non-vide, et le premier contenu du
      Tamis dans C (ce contenu n'apparat plus dans le Tamis);
      Retourne Faux s'il tait vide.
}
VAR
   P    : TPTeteListe;
   Curs : TCurseurListe;

BEGIN { Dpiler_Tamis }
   IF Tamis = NIL
   THEN BEGIN
      Depiler_Tamis := FALSE;
   END { THEN }
   ELSE BEGIN
      Depiler_Tamis := TRUE;
      Debut_Liste (Tamis^.Liste, Curs);
      C := Lire_Liste (Curs);
      Enlever_Liste (Tamis^.Liste, C);
      IF Vide_Liste (Tamis^.Liste)
      THEN BEGIN
         IF Tamis^.Suivant = NIL
         THEN BEGIN
            Lib_Noeud_Mem (Tamis);
         END { THEN }
         ELSE BEGIN
            P := Tamis;
            Tamis := Tamis^.Suivant;
            Lib_Noeud_Mem (P);
         END; { IF }
      END; { IF }
   END; { IF }
END; { Dpiler_Tamis }

{ %%% CONSULTATION NON-DESTRUCTIVE %%% }

FUNCTION Debut_Tamis (Tamis     : TTamis;
                      VAR Curs  : TCurseurTamis) : BOOLEAN;
{
   Place le Curs au dbut du Tamis.

Pr:  Le Tamis existe.

Post: Retourne Faux si le Tamis est vide;
      sinon, retourne Vrai et place le Curs  la premire Liste.
}
BEGIN { Dbut_Tamis }
   IF Tamis = NIL
   THEN BEGIN
      Debut_Tamis := FALSE;
   END { THEN }
   ELSE BEGIN
      Debut_Tamis := TRUE;
      Curs.Tete := Tamis;
      Debut_Liste (Curs.Tete^.Liste, Curs.Curseur);
   END; { IF }
END; { Dbut_Tamis }

FUNCTION Suivant_Tamis (VAR Curs  : TCurseurTamis) : BOOLEAN;
{
   Avance le Curs  la Liste suivante.

Pr:  Le Tamis existe, Curs est valide.

Post: Retourne Vrai s'il existe une liste subsquente, et le Curs associ;
      sinon, retourne Faux et des curseurs NIL.

   NB: La variable Tamis n'est pas ncessaire.
}
BEGIN { Suivant_Tamis }
   Curs.Tete := Curs.Tete^.Suivant;
   IF Curs.Tete = NIL
   THEN BEGIN
      Suivant_Tamis := FALSE;
      Curs.Curseur := NIL;
   END { THEN }
   ELSE BEGIN
      Suivant_Tamis := TRUE;
      Debut_Liste (Curs.Tete^.Liste, Curs.Curseur);
   END; { IF }
END; { Suivant_Tamis }
